#include "engine_game.h"
#include "engine_graphics.h"
#include <rgOpenGL.h>
#include "./arena/arena.h"
#include <SDL/SDL.h>
#include <GL/glu.h>
#include <rg3DRom.h>

GraphicsEngine::GraphicsEngine( GameEngine *ge)
{
	game_engine = ge;
	m_arena = 0;

	init_gl_state();
	set_projection();

	for( int i = 0; i < RPH_TOTAL_CAMERAS; i++)
		m_camera[i] = new rgGLCamera();
	reset_all_cameras();
}

GraphicsEngine::~GraphicsEngine()
{
	clear();
	for( int i = 0; i < RPH_TOTAL_CAMERAS; i++) {
		if( m_camera[i]) { delete (m_camera[i]); m_camera[i] = 0;}
	}
}

void GraphicsEngine::clear()
{
	if( m_arena) { delete m_arena; m_arena = 0;}
	reset_all_cameras();

	init_gl_state();
	set_projection();
}

void GraphicsEngine::reset_all_cameras()
{
	for( int i = 0; i < RPH_TOTAL_CAMERAS; i++) {
		if( m_camera[i]) {
			(m_camera[i])->cs.p = rgVector( 0, 100, 0);
			(m_camera[i])->look( rgVector( 0, 0, 1),
				rgVector( 0, 1, 0));
		}
	}
}

void GraphicsEngine::process()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	(main_cam())->apply();

	glDisable( GL_LIGHTING);
	{
		const float light_level = 0.90;
		// Render reflections
		glPushMatrix();
		glScalef( 1, -1, 1);
		glCullFace( GL_FRONT);
		glColor3f( light_level, light_level, light_level);
		m_arena->draw_boards();
		m_arena->draw_glass();
		glCullFace( GL_BACK);
		glPopMatrix();

		// Render ice-surface
		glEnable( GL_BLEND);
		glColor4f( 1, 1, 1, 0.75);
		m_arena->draw_ice_surface();
		glDisable( GL_BLEND);

		// Render remainder of arena
		glColor3f( light_level, light_level, light_level);
		m_arena->draw_boards();
		m_arena->draw_glass();
	}
	glEnable( GL_LIGHTING);

	SDL_GL_SwapBuffers();
}

void GraphicsEngine::init_gl_state()
{
	glClearColor( 1, 1, 1, 1);

	glFrontFace( GL_CW);
	glCullFace( GL_BACK);
	glEnable( GL_CULL_FACE);

	glEnable( GL_DEPTH_TEST);
	glClearDepth( 1);
	glDepthFunc( GL_LESS);

	glEnable( GL_BLEND);
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable( GL_TEXTURE_2D);
	/*
	glEnable( GL_LIGHTING);
	rgGLLight li;
	li.set_pos( 0, 1, 0, 0);
	li.set_amb( 0.5, 0.5, 0.5, 1);
	li.set_dif( 1, 1, 1, 1);
	li.set_spec( 1, 1, 1, 1);
	li.enable();
	*/
}

void GraphicsEngine::set_projection()
{
	glMatrixMode( GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 90.0f, 1.0f, 4.0f, 4096.0f);
	glMatrixMode( GL_MODELVIEW);
}

int GraphicsEngine::load_arena( const char *fname)
{
	if( !m_arena) m_arena = new phArena();
	const int err = m_arena->load( fname);
	if( err) return err;
	m_arena->prepare_for_rendering();
	return 0;
}
