#include "main_header.h"
#include <SDL/SDL.h>
#include <iostream>
#include "engine_game.h"
#include "things/things.h"

MainData::MainData()
{
	m_screen = 0;
	m_width = 800;
	m_height = 600;
	m_color_depth = 24;
};

int main()
{
	if( krn_initialize_system()) {
		SDL_Quit();
		std::cerr << "Failed to initialize system!\n";
		return -1;
	}

	

	MainData mdata;
	krn_create_screen( mdata.m_screen, mdata.m_width, mdata.m_height,
		mdata.m_color_depth);
	if( !mdata.m_screen) {
		SDL_Quit();
		std::cerr << "Failed to create screen!\n";
		return -2;
	}

	// XXX: Main program loop begins.
	GameEngine *game_engine = new GameEngine( &mdata);
	if( game_engine->load_startup_file( "")) {
		std::cerr << "Failed to load startup file\n";
		SDL_Quit();
		return -3;
	}
	if( game_engine->load_arena( "./media/rph_arena_044.3dr")) {
		std::cerr << "Failed to load arena\n";
		SDL_Quit();
		return -4;
	}

	game_engine->create_thing_manager();
	(game_engine->thing_manager())->create_thing( new phDebugger());

	while( 1) {
		if( krn_check_events( mdata)) break;
		game_engine->process();
	}
	delete game_engine;

	SDL_Quit();
	return 0;
}
