#ifndef THINGS_H
#define THINGS_H

#include <rgStandard.h>
#include <vector>
#include <list>

class GameEngine;
class sk3DRomModel;
class sk3DRomSkelFrame;
class ThingManager;

class phThing
{
public:
	phThing() {
		m_use_gravity = false;
	}

	virtual rgCoordSystem cs() { return m_cs;}
	virtual void process( ThingManager *tm){}
	virtual void draw(){}

protected:
	rgCoordSystem m_cs;

	bool m_use_gravity;
};


class phDebugger : public phThing
{
public:
	void process( ThingManager *tm);
};


class phPlayer : public phThing
{
public:
	phPlayer();
	virtual ~phPlayer();

protected:
	sk3DRomModel *m_mdl;
	sk3DRomSkelFrame *m_skel;
};


class ThingControlBlock
{
public:
	phThing *m_thing;			// The thing

	// Process list thing is in
	std::list <ThingControlBlock *> *m_proc;
	std::list <ThingControlBlock *>::iterator m_iter;

	int m_prev;
	int m_next;

	ThingControlBlock() {
		m_thing = 0;
		m_proc = 0;
		m_prev = -1;
		m_next = -1;
	}
	ThingControlBlock( phThing *thg, std::vector <phThing *> &p_proc) {
		m_thing = thg;
		//m_proc = &p_proc;
		//if( !m_proc) return;
		//m_proc->push_back( m_thing);
		m_prev = -1;
		m_next = -1;
	}
};

#define TM_MAX_PROC_LIST	16
#define TM_MAX_THINGS		16000
class ThingManager
{
public:

	ThingManager( GameEngine *ge);
	~ThingManager();

	void clear();
	int create_thing( phThing *thg);

	void process();
	GameEngine *ge() { return m_game_engine;}	

protected:
	void process_list( std::list <ThingControlBlock *> &p_proc);

	//std::vector <ThingControlBlock> m_thing;
	GameEngine *m_game_engine;

	int m_max_things;
	int m_thing_count;
	ThingControlBlock *m_things;
	std::list <ThingControlBlock *> m_proc[ TM_MAX_PROC_LIST];

	int m_thing_head;
	int m_thing_free;

	void init_things();
	void move_thing_to_proc( ThingControlBlock &tcb,
		std::list <ThingControlBlock *> &p_proc);
	void remove_thing_from_proc( ThingControlBlock &tcb);

};


#endif
